package org.eclipse.jdt.internal.compiler.tool;

package .tmp;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import javax.annotation.processing.Processor;
import javax.tools.Diagnostic;
import javax.tools.DiagnosticListener;
import javax.tools.JavaFileManager;
import javax.tools.JavaFileObject;
import javax.tools.StandardJavaFileManager;
import javax.tools.StandardLocation;
import org.eclipse.jdt.core.compiler.CategorizedProblem;
import org.eclipse.jdt.core.compiler.CharOperation;
import org.eclipse.jdt.core.compiler.CompilationProgress;
import org.eclipse.jdt.internal.compiler.ClassFile;
import org.eclipse.jdt.internal.compiler.CompilationResult;
import org.eclipse.jdt.internal.compiler.IErrorHandlingPolicy;
import org.eclipse.jdt.internal.compiler.IProblemFactory;
import org.eclipse.jdt.internal.compiler.batch.ClasspathJsr199;
import org.eclipse.jdt.internal.compiler.batch.CompilationUnit;
import org.eclipse.jdt.internal.compiler.batch.FileSystem;
import org.eclipse.jdt.internal.compiler.batch.Main;
import org.eclipse.jdt.internal.compiler.tool.EclipseFileManager;
import org.eclipse.jdt.internal.compiler.util.Messages;
import org.eclipse.jdt.internal.compiler.util.SuffixConstants;

public class EclipseCompilerImpl extends Main {
  private static final CompilationUnit[] NO_UNITS = new CompilationUnit[0];
  
  private HashMap<CompilationUnit, JavaFileObject> javaFileObjectMap;
  
  Iterable<? extends JavaFileObject> compilationUnits;
  
  public JavaFileManager fileManager;
  
  protected Processor[] processors;
  
  public DiagnosticListener<? super JavaFileObject> diagnosticListener;
  
  public EclipseCompilerImpl(PrintWriter out, PrintWriter err, boolean systemExitWhenFinished) {
    super(out, err, systemExitWhenFinished, null, null);
  }
  
  public boolean call() {
    try {
      if (this.proceed) {
        this.globalProblemsCount = 0;
        this.globalErrorsCount = 0;
        this.globalWarningsCount = 0;
        this.globalTasksCount = 0;
        this.exportedClassFilesCounter = 0;
        performCompilation();
      } 
    } catch (IllegalArgumentException e) {
      this.logger.logException(e);
      if (this.systemExitWhenFinished) {
        cleanup();
        System.exit(-1);
      } 
      return false;
    } catch (RuntimeException e) {
      this.logger.logException(e);
      return false;
    } finally {
      cleanup();
    } 
    if (this.globalErrorsCount == 0)
      return true; 
    return false;
  }
  
  private void cleanup() {
    this.logger.flush();
    this.logger.close();
    this.processors = null;
    try {
      if (this.fileManager != null)
        this.fileManager.flush(); 
    } catch (IOException iOException) {}
  }
  
  public CompilationUnit[] getCompilationUnits() {
    if (this.compilationUnits == null)
      return NO_UNITS; 
    ArrayList<CompilationUnit> units = new ArrayList<>();
    for (JavaFileObject javaFileObject : this.compilationUnits) {
      if (javaFileObject.getKind() != JavaFileObject.Kind.SOURCE)
        throw new IllegalArgumentException(); 
      String name = javaFileObject.getName();
      name = name.replace('\\', '/');
      Object object = new Object(this, null, 
          name, 
          null, javaFileObject);
      units.add(object);
      this.javaFileObjectMap.put(object, javaFileObject);
    } 
    CompilationUnit[] result = new CompilationUnit[units.size()];
    units.toArray(result);
    return result;
  }
  
  public IErrorHandlingPolicy getHandlingPolicy() {
    return (IErrorHandlingPolicy)new Object(this);
  }
  
  public IProblemFactory getProblemFactory() {
    return (IProblemFactory)new Object(this);
  }
  
  protected void initialize(PrintWriter outWriter, PrintWriter errWriter, boolean systemExit, Map<String, String> customDefaultOptions, CompilationProgress compilationProgress) {
    super.initialize(outWriter, errWriter, systemExit, customDefaultOptions, compilationProgress);
    this.javaFileObjectMap = new HashMap<>();
  }
  
  protected void initializeAnnotationProcessorManager() {
    super.initializeAnnotationProcessorManager();
    if (this.batchCompiler.annotationProcessorManager != null && this.processors != null) {
      this.batchCompiler.annotationProcessorManager.setProcessors((Object[])this.processors);
    } else if (this.processors != null) {
      throw new UnsupportedOperationException("Cannot handle annotation processing");
    } 
  }
  
  public void outputClassFiles(CompilationResult unitResult) {
    if (unitResult != null && (!unitResult.hasErrors() || this.proceedOnError)) {
      ClassFile[] classFiles = unitResult.getClassFiles();
      boolean generateClasspathStructure = this.fileManager.hasLocation(StandardLocation.CLASS_OUTPUT);
      String currentDestinationPath = this.destinationPath;
      File outputLocation = null;
      if (currentDestinationPath != null) {
        outputLocation = new File(currentDestinationPath);
        outputLocation.mkdirs();
      } 
      for (int i = 0, fileCount = classFiles.length; i < fileCount; i++) {
        ClassFile classFile = classFiles[i];
        char[] filename = classFile.fileName();
        int length = filename.length;
        char[] relativeName = new char[length + 6];
        System.arraycopy(filename, 0, relativeName, 0, length);
        System.arraycopy(SuffixConstants.SUFFIX_class, 0, relativeName, length, 6);
        CharOperation.replace(relativeName, '/', File.separatorChar);
        String relativeStringName = new String(relativeName);
        if (this.compilerOptions.verbose)
          this.out.println(
              Messages.bind(
                Messages.compilation_write, 
                (Object[])new String[] { String.valueOf(this.exportedClassFilesCounter + 1), 
                  relativeStringName })); 
        try {
          Exception exception2;
          JavaFileObject javaFileForOutput = 
            this.fileManager.getJavaFileForOutput(
              StandardLocation.CLASS_OUTPUT, 
              new String(filename), 
              JavaFileObject.Kind.CLASS, 
              this.javaFileObjectMap.get(unitResult.compilationUnit));
          if (generateClasspathStructure)
            if (currentDestinationPath != null) {
              int index = CharOperation.lastIndexOf(File.separatorChar, relativeName);
              if (index != -1) {
                File currentFolder = new File(currentDestinationPath, relativeStringName.substring(0, index));
                currentFolder.mkdirs();
              } 
            } else {
              String path = javaFileForOutput.toUri().getPath();
              int index = path.lastIndexOf('/');
              if (index != -1) {
                File file = new File(path.substring(0, index));
                file.mkdirs();
              } 
            }  
          Exception exception1 = null;
        } catch (IOException e) {
          this.logger.logNoClassFileCreated(currentDestinationPath, relativeStringName, e);
        } 
        this.logger.logClassFile(
            generateClasspathStructure, 
            currentDestinationPath, 
            relativeStringName);
        this.exportedClassFilesCounter++;
      } 
      this.batchCompiler.lookupEnvironment.releaseClassFiles(classFiles);
    } 
  }
  
  protected void setPaths(ArrayList bootclasspaths, String sourcepathClasspathArg, ArrayList sourcepathClasspaths, ArrayList classpaths, ArrayList extdirsClasspaths, ArrayList endorsedDirClasspaths, String customEncoding) {
    ArrayList<FileSystem.Classpath> fileSystemClasspaths = new ArrayList<>();
    EclipseFileManager eclipseJavaFileManager = null;
    StandardJavaFileManager standardJavaFileManager = null;
    JavaFileManager javaFileManager = null;
    boolean havePlatformPaths = false;
    boolean haveClassPaths = false;
    if (this.fileManager instanceof EclipseFileManager)
      eclipseJavaFileManager = (EclipseFileManager)this.fileManager; 
    if (this.fileManager instanceof StandardJavaFileManager)
      standardJavaFileManager = (StandardJavaFileManager)this.fileManager; 
    javaFileManager = this.fileManager;
    if (eclipseJavaFileManager != null && (
      eclipseJavaFileManager.flags & 0x4) == 0 && (
      eclipseJavaFileManager.flags & 0x2) != 0)
      fileSystemClasspaths.addAll(handleEndorseddirs(null)); 
    Iterable<? extends File> location = null;
    if (standardJavaFileManager != null) {
      location = standardJavaFileManager.getLocation(StandardLocation.PLATFORM_CLASS_PATH);
      if (location != null)
        for (File file : location) {
          FileSystem.Classpath classpath = FileSystem.getClasspath(
              file.getAbsolutePath(), 
              null, 
              null, this.options);
          if (classpath != null) {
            fileSystemClasspaths.add(classpath);
            havePlatformPaths = true;
          } 
        }  
    } else if (javaFileManager != null) {
      ClasspathJsr199 classpathJsr199 = new ClasspathJsr199(this.fileManager, StandardLocation.PLATFORM_CLASS_PATH);
      fileSystemClasspaths.add(classpathJsr199);
      havePlatformPaths = true;
    } 
    if (eclipseJavaFileManager != null && (
      eclipseJavaFileManager.flags & 0x1) == 0 && (
      eclipseJavaFileManager.flags & 0x2) != 0)
      fileSystemClasspaths.addAll(handleExtdirs(null)); 
    if (standardJavaFileManager != null) {
      location = standardJavaFileManager.getLocation(StandardLocation.SOURCE_PATH);
      if (location != null)
        for (File file : location) {
          FileSystem.Classpath classpath = FileSystem.getClasspath(
              file.getAbsolutePath(), 
              null, 
              null, this.options);
          if (classpath != null)
            fileSystemClasspaths.add(classpath); 
        }  
      location = standardJavaFileManager.getLocation(StandardLocation.CLASS_PATH);
      if (location != null)
        for (File file : location) {
          FileSystem.Classpath classpath = FileSystem.getClasspath(
              file.getAbsolutePath(), 
              null, 
              null, this.options);
          if (classpath != null) {
            fileSystemClasspaths.add(classpath);
            haveClassPaths = true;
          } 
        }  
    } else if (javaFileManager != null) {
      FileSystem.Classpath classpath = null;
      if (this.fileManager.hasLocation(StandardLocation.SOURCE_PATH)) {
        ClasspathJsr199 classpathJsr1991 = new ClasspathJsr199(this.fileManager, StandardLocation.SOURCE_PATH);
        fileSystemClasspaths.add(classpathJsr1991);
      } 
      ClasspathJsr199 classpathJsr199 = new ClasspathJsr199(this.fileManager, StandardLocation.CLASS_PATH);
      fileSystemClasspaths.add(classpathJsr199);
      haveClassPaths = true;
    } 
    if (this.checkedClasspaths == null) {
      if (!havePlatformPaths)
        fileSystemClasspaths.addAll(handleBootclasspath(null, null)); 
      if (!haveClassPaths)
        fileSystemClasspaths.addAll(handleClasspath(null, null)); 
    } 
    fileSystemClasspaths = FileSystem.ClasspathNormalizer.normalize(fileSystemClasspaths);
    int size = fileSystemClasspaths.size();
    if (size != 0) {
      this.checkedClasspaths = new FileSystem.Classpath[size];
      int i = 0;
      for (FileSystem.Classpath classpath : fileSystemClasspaths)
        this.checkedClasspaths[i++] = classpath; 
    } 
  }
  
  protected void loggingExtraProblems() {
    super.loggingExtraProblems();
    for (Iterator<CategorizedProblem> iterator = this.extraProblems.iterator(); iterator.hasNext(); ) {
      CategorizedProblem problem = iterator.next();
      if (this.diagnosticListener != null)
        this.diagnosticListener.report((Diagnostic<? extends JavaFileObject>)new Object(this, problem)); 
    } 
  }
}
